
//  @(#)header.C 1.2 93/12/16
//
//  Various routines associated with the header popup.
//
//  Copyright (c) Rich Burridge.    All rights reserved.
//
//  Permission is given to distribute these sources, as long as the
//  copyright messages are not removed, and no monies are exchanged.
//
//  No responsibility is taken for any errors or inaccuracies inherent
//  either to the comments or the code of this program, but if
//  reported to me then an attempt will be made to fix them.

#include "mptool.h"
#include <Xm/List.h>


void Header::
do_hdrs_add(void)
{
  char buf[MAXLINE] ;
  int n ;

  STRCPY(buf, x.get_str_value(F_HDRS, WI_HDR_LIST_ADD)) ;
  n = strlen(buf) ;
  if (buf == NULL || !n)
    x.notice(hwarr[WI_HEADER_FRAME], mptool_strs[(int) S_NOTHING]) ;
  else
    { 
      if (buf[n-1] != ':') STRCAT(buf, ":") ;
      x.add_list_item(hwarr[WI_HDR_LIST], 0, buf) ;
      x.set_str_value(F_HDRS, WI_HDR_LIST_ADD, "") ;
      x.set_list_position(hwarr[WI_HDR_LIST], 0) ;
    }
}


void Header::
do_hdrs_apply(void)
{
  char *liststr ;
  int found, i, j ;
  int nrows = x.get_scroll_nrows(hwarr[WI_HDR_LIST]) ;

  opt.allhdrs = x.get_bool_value(F_HDRS, WI_HDRS_ALL) ;

  for (i = 0; i < opt.remhdr_num; i++) FREE(opt.remhdrs[i]) ;
  opt.remhdr_num = 0 ;
  for (i = 0; i < MAXSTDHDRS; i++)
    {
      found = FALSE ;
      for (j = 0; j < nrows && found == FALSE; j++)
        if (EQUAL(hdrs[i], get_list_str(j))) found = TRUE ;
      if (found == FALSE)
        opt.get_hdr_arg(hdrs[i], opt.remhdrs, &opt.remhdr_num) ;
    }
 
  for (i = 0; i < opt.addhdr_num; i++) FREE(opt.addhdrs[i]) ;
  opt.addhdr_num = 0 ;
  for (i = 0; i < nrows; i++)
    {
      found = FALSE ;
      liststr = get_list_str(i) ;
      for (j = 0; j < MAXSTDHDRS && found == FALSE; j++)
        if (EQUAL(liststr, hdrs[j])) found = TRUE ;
      if (found == FALSE)
        opt.get_hdr_arg(liststr, opt.addhdrs, &opt.addhdr_num) ;
    }
}


void Header::
do_hdrs_dismiss(void)
{
  XtUnmapWidget(hpopup) ;
  XtUnmanageChild(hpopup) ;
}


void Header::
do_hdrs_remove(void)
{
  int i, n, *pos_list ;

  if (XmListGetSelectedPos(hwarr[WI_HDR_LIST], &pos_list, &n))
    {
      for (i = 0; i < n; i++)
        XmListDeleteItemsPos(hwarr[WI_HDR_LIST], 1, pos_list[i]) ;
      x.set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, False) ;
    }
  else x.notice(hwarr[WI_HEADER_FRAME], mptool_strs[(int) S_NONE]) ;
}


void Header::
do_hdrs_reset(void)
{
  int i, pos ;

  x.set_bool_value(F_HDRS, WI_HDRS_ALL, opt.allhdrs) ;
  XmListDeleteAllItems(hwarr[WI_HDR_LIST]) ;
  for (i = 0; i < MAXSTDHDRS; i++)
    x.add_list_item(hwarr[WI_HDR_LIST], 0, hdrs[i]) ;

  for (i = 0; i < opt.addhdr_num; i++)
    if (x.item_exists(hwarr[WI_HDR_LIST], opt.addhdrs[i]) == 0)
      x.add_list_item(hwarr[WI_HDR_LIST], 0, opt.addhdrs[i]) ;

  for (i = 0; i < opt.remhdr_num; i++)
    if ((pos = x.item_exists(hwarr[WI_HDR_LIST], opt.remhdrs[i])) != 0)
      x.delete_list_item(hwarr[WI_HDR_LIST], pos) ;
  do_toggle_hdr_list() ;
}


void Header::
do_show_headers(void)
{
  if (!hpopup)
    {
      hpopup = tu_header_frame_widget("headers", base.warr[WI_MAIN_FRAME],
                                      &hwarr) ;
      x.add_delete_callback(hpopup) ;
      XtAddCallback(hwarr[WI_HDR_LIST], XmNbrowseSelectionCallback,
                (XtCallbackProc) Header::hdr_select, NULL) ;
      XtAddCallback(hwarr[WI_HDR_LIST_ADD], XmNmodifyVerifyCallback,
                                (XtCallbackProc) Header::hdr_text, 0) ;

      help.add_help(hwarr[WI_HEADER_FORM],       wi_header_frame_help) ;
      help.add_help(hwarr[WI_HDRS_ALL],          wi_headers_all_help) ;
      help.add_help(hwarr[WI_HDR_LIST],          wi_hdr_list_help) ;
      help.add_help(hwarr[WI_HDR_LIST_ADD],      wi_hdr_add_help) ;
      help.add_help(hwarr[WI_HDR_ADD_LABEL],     wi_hdr_add_help) ;
      help.add_help(hwarr[WI_HDR_APPLY],         wi_hdr_apply_help) ;
      help.add_help(hwarr[WI_HDR_DISMISS],       wi_hdr_dismiss_help) ;
      help.add_help(hwarr[WI_HDR_ADD_BUTTON],    wi_hdr_add_help) ;
      help.add_help(hwarr[WI_HDR_REMOVE_BUTTON], wi_hdr_remove_help) ;
      help.add_help(hwarr[WI_HDR_RESET],         wi_hdr_reset_help) ;
      x.set_active(F_HDRS, WI_HDR_ADD_BUTTON,    False) ;
      x.set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, False) ;
      do_hdrs_reset() ;                                 // Headers to print.
    }
  x.show_popup(hpopup) ;
}


void Header::
do_toggle_hdr_list(void)
{
  int n, *pos_list ;
  char *str    = x.get_str_value(F_HDRS, WI_HDR_LIST_ADD) ;
  int value    = x.get_bool_value(F_HDRS, WI_HDRS_ALL) ;
  int selected = XmListGetSelectedPos(hwarr[WI_HDR_LIST], &pos_list, &n) ;

  x.set_active(F_HDRS, WI_HDR_LIST,          !value) ;
  x.set_active(F_HDRS, WI_HDR_LIST_ADD,      !value) ;
  x.set_active(F_HDRS, WI_HDR_ADD_LABEL,     !value) ;
  x.set_active(F_HDRS, WI_HDR_ADD_BUTTON,    False) ;
  x.set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, False) ;

  if (!value && str && *str) x.set_active(F_HDRS, WI_HDR_ADD_BUTTON,    True) ;
  if (!value && selected)    x.set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, True) ;
}


char * Header::
get_list_str(int n)
{
  char *label ;
  int nrows = x.get_scroll_nrows(hwarr[WI_HDR_LIST]) ;
  XmString *strlist ;

  XtVaGetValues(hwarr[WI_HDR_LIST], XmNitems, &strlist, 0) ;
  XmStringGetLtoR(strlist[n], XmSTRING_DEFAULT_CHARSET, &label) ;
  return(label) ;
}


Header::
Header()
{
  int i ;

  hwarr  = NULL ;                // No widgets defined initially.
  hpopup = NULL ;                // No header popup window initially.
  for (i = 0; i < MAXSTDHDRS; i++) hdrs[i] = NULL ;
}


void Header::
hdr_select(Widget, XtPointer, XmListCallbackStruct *)
{
  x.set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, True) ;
}


void Header::
hdr_text(Widget, XtPointer, XmTextVerifyCallbackStruct *)
{
  x.set_active(F_HDRS, WI_HDR_ADD_BUTTON, True) ;
}


void Header::
init_headers(void)
{
  int i = 0 ;

  u.read_str(&hdrs[i++], APP_FROMHDR) ;
  u.read_str(&hdrs[i++], APP_TOHDR) ;
  u.read_str(&hdrs[i++], CCHDR) ;
  u.read_str(&hdrs[i++], CONTENT_LEN) ;
  u.read_str(&hdrs[i++], DATEHDR) ;
  u.read_str(&hdrs[i++], FROMHDR) ;
  u.read_str(&hdrs[i++], FROM_HDR) ;
  u.read_str(&hdrs[i++], NEWSGROUPHDR) ;
  u.read_str(&hdrs[i++], NEWSGROUPSHDR) ;
  u.read_str(&hdrs[i++], REPLYHDR) ;
  u.read_str(&hdrs[i++], SUBJECTHDR) ;
  u.read_str(&hdrs[i++], TOHDR) ;
}
