
//  @(#)print.C 1.3 93/12/22
//
//  Various routines associated with the printing popup.
//
//  Copyright (c) Rich Burridge.    All rights reserved.
//
//  Permission is given to distribute these sources, as long as the
//  copyright messages are not removed, and no monies are exchanged.
//
//  No responsibility is taken for any errors or inaccuracies inherent
//  either to the comments or the code of this program, but if
//  reported to me then an attempt will be made to fix them.

#include "mptool.h"


void Print::
check_int_value(int *value, char *str,
                int minval, int maxval, int defval, int n)
{
  char m[MAXLINE] ;

  *value = x.get_int_value(F_PRINT, n) ;
  if (*value < minval || *value > maxval)
    {
      SPRINTF(m, str, defval) ;
      x.notice(pwarr[WI_PRINTING_FRAME], m) ;
      *value = defval ;
      x.set_int_value(F_PRINT, n, *value) ;
    }
}

void Print::
do_printing_apply(void)
{
  check_int_value(&opt.numcols, mptool_strs[(int) S_INVCOL],
                  1, MAXLINELENGTH, DEFCOLUMNS, WI_PRINT_COL) ;
  check_int_value(&opt.tabsize, mptool_strs[(int) S_INVTAB],
                  1, MAXLINELENGTH, TABSIZE, WI_PRINT_TAB) ;
  check_int_value(&opt.bmargin, mptool_strs[(int) S_BMARGIN],
                  0, MAXMARGIN, DEFMARGIN, WI_PRINT_BMARGIN) ;
  check_int_value(&opt.lmargin, mptool_strs[(int) S_LMARGIN], 
                  0, MAXMARGIN, DEFMARGIN, WI_PRINT_LMARGIN) ;
  check_int_value(&opt.rmargin, mptool_strs[(int) S_RMARGIN], 
                  0, MAXMARGIN, DEFMARGIN, WI_PRINT_RMARGIN) ;
  check_int_value(&opt.tmargin, mptool_strs[(int) S_TMARGIN], 
                  0, MAXMARGIN, DEFMARGIN, WI_PRINT_TMARGIN) ;

  opt.banner     = x.get_bool_value(F_PRINT, WI_PRINT_HEADER) ;
  opt.wrapwords  = x.get_bool_value(F_PRINT, WI_PRINT_WRAP) ;
  opt.flip       = x.get_bool_value(F_PRINT, WI_PRINT_FLIP) ;
  opt.forcepage  = x.get_bool_value(F_PRINT, WI_PRINT_FORCEPAGE) ;
  opt.number     = x.get_bool_value(F_PRINT, WI_PRINT_NUMBER) ;
  opt.tumble     = x.get_bool_value(F_PRINT, WI_PRINT_REVERSE) ;
}       


void Print::
do_printing_dismiss(void)
{       
  XtUnmapWidget(ppopup) ;
  XtUnmanageChild(ppopup) ;
}       


void Print::
do_printing_reset(void)
{       
  x.set_int_value(F_PRINT,  WI_PRINT_COL,       opt.numcols) ;
  x.set_int_value(F_PRINT,  WI_PRINT_TAB,       opt.tabsize) ;
  x.set_int_value(F_PRINT,  WI_PRINT_BMARGIN,   opt.bmargin) ;
  x.set_int_value(F_PRINT,  WI_PRINT_LMARGIN,   opt.lmargin) ;
  x.set_int_value(F_PRINT,  WI_PRINT_RMARGIN,   opt.rmargin) ;
  x.set_int_value(F_PRINT,  WI_PRINT_TMARGIN,   opt.tmargin) ;
  x.set_bool_value(F_PRINT, WI_PRINT_HEADER,    opt.banner) ;
  x.set_bool_value(F_PRINT, WI_PRINT_WRAP,      opt.wrapwords) ;
  x.set_bool_value(F_PRINT, WI_PRINT_FLIP,      opt.flip) ;
  x.set_bool_value(F_PRINT, WI_PRINT_FORCEPAGE, opt.forcepage) ;
  x.set_bool_value(F_PRINT, WI_PRINT_NUMBER,    opt.number) ;
  x.set_bool_value(F_PRINT, WI_PRINT_REVERSE,   opt.tumble) ;
}


void Print::
do_show_printing(void)
{
  if (!ppopup)
    {
      ppopup = tu_printing_frame_widget("printing", base.warr[WI_MAIN_FRAME],
                                        &pwarr) ;
      x.add_delete_callback(ppopup) ;
      help.add_help(pwarr[WI_PRINTING_FORM],     wi_props_frame_help) ;
      help.add_help(pwarr[WI_PRINT_COL_LABEL],   wi_props_col_help) ;
      help.add_help(pwarr[WI_PRINT_COL],         wi_props_col_help) ;
      help.add_help(pwarr[WI_PRINT_TAB_LABEL],   wi_props_tab_help) ;
      help.add_help(pwarr[WI_PRINT_TAB],         wi_props_tab_help) ;
      help.add_help(pwarr[WI_PRINT_BMARGIN],     wi_props_margin_help) ;
      help.add_help(pwarr[WI_PRINT_LMARGIN],     wi_props_margin_help) ;
      help.add_help(pwarr[WI_PRINT_RMARGIN],     wi_props_margin_help) ;
      help.add_help(pwarr[WI_PRINT_TMARGIN],     wi_props_margin_help) ;
      help.add_help(pwarr[WI_PRINT_BMLABEL],     wi_props_margin_help) ;
      help.add_help(pwarr[WI_PRINT_LMLABEL],     wi_props_margin_help) ;
      help.add_help(pwarr[WI_PRINT_RMLABEL],     wi_props_margin_help) ;
      help.add_help(pwarr[WI_PRINT_TMLABEL],     wi_props_margin_help) ;
      help.add_help(pwarr[WI_PRINT_MLABEL],      wi_props_margin_help) ;
      help.add_help(pwarr[WI_PRINT_HEADER],      wi_opt_header_help) ;
      help.add_help(pwarr[WI_PRINT_WRAP],        wi_props_wrap_help) ;
      help.add_help(pwarr[WI_PRINT_FLIP],        wi_props_flip_help) ;
      help.add_help(pwarr[WI_PRINT_FORCEPAGE],   wi_props_forcepage_help) ;
      help.add_help(pwarr[WI_PRINT_NUMBER],      wi_props_number_help) ;
      help.add_help(pwarr[WI_PRINT_REVERSE],     wi_props_reverse_help) ;
      help.add_help(pwarr[WI_PRINT_APPLY],       wi_props_apply_help) ;
      help.add_help(pwarr[WI_PRINT_RESET],       wi_props_reset_help) ;
      help.add_help(pwarr[WI_PRINT_DISMISS],     wi_props_dismiss_help) ;
      do_printing_reset() ;
    } 
  x.show_popup(ppopup) ;
}


Print::
Print()
{  
  pwarr = NULL ;           // No widgets defined initially.
  ppopup = NULL ;          // No printing popup window initially.
}
